<?php

namespace App\Service;

use App\Exception;

class DispatchService {
    
    private $connection;
    private $util;
    private $branchFactory;

    public function __construct (\Doctrine\DBAL\Connection $connection,
                                 \App\Util\Util $util,
                                 \App\Factory\BranchFactory $branchFactory)
    {
        $this->connection    = $connection;
        $this->util          = $util;
        $this->branchFactory = $branchFactory;
    }
    
    const GET_PENDING_DISPATCH_ORDERS = "
        SELECT C.CDFILIAL, C.NRCOMANDA, C.NRCOMANDAEXT, C.IDSTCOMANDA, I.CDINTEGRACAO,INTF.IDTOKEN
          FROM COMANDAVEN C, INTEGRACAO I, INTEGRACAOFILI INTF
         WHERE I.IDINTEGRACAO    = 'ANT'
           AND INTF.CDFILIAL     =  C.CDFILIAL
           AND INTF.CDINTEGRACAO = I.CDINTEGRACAO
           AND C.CDFILIAL        IN(?)
           AND C.NRCOMANDA       IN(?)
           AND C.NRCOMANDAEXT    IN(?)
           AND C.IDORGCMDVENDA   = 'DLV_ANT'
           AND INTF.IDTOKENINTGERALFILI IN(?)
    ";

    public function getPendingDispatchOrders($cdFilialArr, $nrComandaArr, $nrComandaExtArr, $merchantId)
    {
        $params = [
            $cdFilialArr,
            $nrComandaArr,
            $nrComandaExtArr,
            $merchantId
        ];
    
        $types = [\Doctrine\DBAL\Connection::PARAM_STR_ARRAY, 
                  \Doctrine\DBAL\Connection::PARAM_STR_ARRAY, 
                  \Doctrine\DBAL\Connection::PARAM_STR_ARRAY,
                  \Doctrine\DBAL\Connection::PARAM_STR_ARRAY];

        return $this->connection->fetchAll(self::GET_PENDING_DISPATCH_ORDERS, $params, $types);
    }

    public function registerPendingDispatchOrder($cdFilial, $nrComanda, $nrComandaExt, $reference, $orderType, $merchantId)
    {
        $pendingDispatchFile = $this->util->getPendingOrdersFile();
        if(!file_exists($pendingDispatchFile)) {
            file_put_contents($pendingDispatchFile, json_encode([], JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }
        
        $pendingDispatchFileContent = file_get_contents($pendingDispatchFile);
        $pendingDispatchData        = json_decode($pendingDispatchFileContent, true);

        $key = "$cdFilial|$nrComanda|$nrComandaExt";
        if(!isset($pendingDispatchData[$key])) {
            $pendingDispatchData[$key] = [
                'cdFilial'     => $cdFilial,
                'merchantId'   => $merchantId,
                'nrComanda'    => $nrComanda,
                'nrComandaExt' => $nrComandaExt,
                'reference'    => $reference,
                'orderType'    => $orderType,
                'dispatchedAt' => '',
                'delivered'    => false,
                'canceled'     => false,
                'createdAt'    => time()
            ];        
        }

        file_put_contents($pendingDispatchFile, json_encode($pendingDispatchData, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
    }
    
    public function getFilialToken($cdFilial, $merchantId)
    {
        return $this->branchFactory->getFilialToken($cdFilial, $merchantId);
    }
}
