<?php

namespace App\Service;

use App\Exception\SqlException as SqlException;
use App\Exception\ParamsException as ParamsException;

class ProductService {
    
    private $productFactory;
    private $util;
    
    public function __construct (\App\Factory\ProductFactory $productFactory,
                                 \App\Util\Util $util)
    {
        
        $this->productFactory = $productFactory;
        $this->util           = $util;
    }

    public function processProduct ($generalParams, $product, $order, &$ITCOMANDAVEN_SEQ)
    {

        if((isset($product["externalId"]) && !empty($product["externalId"])) || $product["price"] != 0) { //Produto com código externo
            $processedProduct = $this->processProductWithExtCode($generalParams, $product, $order, $ITCOMANDAVEN_SEQ);    

        } else { //Produto sem código externo
            $processedProduct = $this->processProductWithoutExtCode($generalParams, $product, $order, $ITCOMANDAVEN_SEQ);    
        }
        
        return $processedProduct;
    }
    
    private function processProductWithExtCode ($generalParams, $product, $order, &$ITCOMANDAVEN_SEQ)
    {
        $productItems = [];

        if (!isset($product["externalId"]) || empty($product["externalId"]))
                throw new ParamsException("
                    O produto {$product['name']} não possui código externo e ele é obrigatório.<br>
                    Verifique no Odhen Retail o código externo deste produto e faça a correção na plataforma Anota Aí.<br>");
        
        $productDetails = $this->productFactory->getProduct($product["externalId"]);

        if (!isset($productDetails) || empty($productDetails))
                throw new ParamsException("
                    O produto {$product['name']} não possui o código externo {$product['externalId']} cadastrado no Sistema Teknisa .<br>
                    Verifique no Odhen Retail o código externo deste produto e faça as devidas correções.<br>");


        $productDetails = array_merge($productDetails, $product);
        
        $txProdComVenPai = "";

        for($i = 0; $i < $product["quantity"]; $i++) {
            $nrSeqProdComPai = str_pad((string) ($ITCOMANDAVEN_SEQ + 1), 6, '0', STR_PAD_LEFT);
            $ITCOMANDAVEN_SEQ++;

            if(!empty($productDetails["observation"])){
                $txProdComVenPai = " ".$productDetails["observation"].";";
            }

            $pruductSimpleTemplate = [
                "CDFILIAL"         => $generalParams["CDFILIAL"],
                "NRVENDAREST"      => $order["NRVENDAREST"],
                "NRCOMANDA"        => $order["NRCOMANDA"],
                "NRPRODCOMVEN"     => $nrSeqProdComPai,
                "CDPRODUTO"        => $productDetails["CDPRODUTO"],
                "QTPRODCOMVEN"     => 1,
                "VRPRECCOMVEN"     => $productDetails["price"], //bcdiv($productDetails["price"], 100, 2), 
                "TXPRODCOMVEN"     => "",
                "IDSTPRCOMVEN"     => $generalParams["IDCONTROPROD"] === "S" ? "1" : "3",
                "VRDESCCOMVEN"     => 0,
                "NRLUGARMESA"      => "01",
                "IDPRODIMPFIS"     => "N",
                "CDLOJA"           => $generalParams["CDLOJA"],
                "VRACRCOMVEN"      => 0,
                "NRSEQPRODCOM"     => null,
                "NRSEQPRODCUP"     => $generalParams["NRSEQPRODCUP"],
                "VRPRECCLCOMVEN"   => 0,
                "CDCAIXACOLETOR"   => $generalParams["CDCAIXA"],
                "CDPRODPROMOCAO"   => null,
                "CDVENDEDOR"       => $generalParams["CDVENDEDOR"],
                "NRINSCRCONS"      => "",
                "DSOBSPEDDIGCMD"   => null,
                "ITEM_OBSERVATION" => []
            ];

            foreach ($product["subItems"] as $subItem) {
                
                if($subItem["price"] > 0) {
                    
                    if (!isset($subItem["externalId"]) || empty($subItem["externalId"])) 
                        throw new ParamsException("
                            O produto {$subItem['name']} não possui código externo e ele é obrigatório.<br>
                            Verifique no Odhen Retail o código externo deste produto e faça a correção na plataforma Anota Aí.<br>");
                    
                    $validateProd = $this->productFactory->getProduct($subItem["externalId"]);
                    
                    if (!isset($validateProd) || empty($validateProd))
                        throw new ParamsException("
                            O produto {$subItem['name']} não possui o código externo {$subItem['externalId']} cadastrado no Sistema Teknisa .<br>
                            Verifique no Odhen Retail o código externo deste produto e faça as devidas correções.<br>");

                    for($y = 0; $y < $subItem["quantity"]; $y++) {
                        $subItemDetails = array_merge($validateProd, $subItem);
                        
                        $subItemDetails["NRPRODCOMVEN"] = str_pad((string) ($ITCOMANDAVEN_SEQ + 1), 6, '0', STR_PAD_LEFT);
                        $ITCOMANDAVEN_SEQ++;
                        
                        $txProdComVenIt = "";
                        if(!empty($subItemDetails["observation"])){
                            $txProdComVenIt = " ".$subItemDetails["observation"].";";
                        }
                        
                        $subItemReturn = [
                            "CDFILIAL"          => $generalParams["CDFILIAL"],
                            "NRVENDAREST"       => $order["NRVENDAREST"],
                            "NRCOMANDA"         => $order["NRCOMANDA"],
                            "NRPRODCOMVEN"      => $subItemDetails["NRPRODCOMVEN"],
                            "CDPRODUTO"         => $subItemDetails["CDPRODUTO"],
                            "QTPRODCOMVEN"      => 1,
                            "VRPRECCOMVEN"      => $subItemDetails["price"], //bcdiv($subItemDetails["price"], 100, 2),
                            "TXPRODCOMVEN"      => $txProdComVenIt,
                            "IDSTPRCOMVEN"      => $generalParams["IDCONTROPROD"] === "S" ? "1" : "3",
                            "VRDESCCOMVEN"      => 0,
                            "NRLUGARMESA"       => "01",
                            "IDPRODIMPFIS"      => "N",
                            "CDLOJA"            => $generalParams["CDLOJA"],
                            "VRACRCOMVEN"       => 0,
                            "NRSEQPRODCOM"      => $this->util->formatNrSeqProdCom($nrSeqProdComPai),
                            "NRSEQPRODCUP"      => $generalParams["NRSEQPRODCUP"],
                            "VRPRECCLCOMVEN"    => 0,
                            "CDCAIXACOLETOR"    => $generalParams["CDCAIXA"],
                            "CDPRODPROMOCAO"    => $productDetails["CDPRODUTO"],
                            "CDVENDEDOR"        => $generalParams["CDVENDEDOR"],
                            "NRINSCRCONS"       => "",
                            "DSOBSPEDDIGCMD"    => null,
                            "ITEM_OBSERVATION"  => [] //
                        ];
                        
                        array_push($productItems, $subItemReturn);
                    }
                    
                    //$pruductSimpleTemplate["NRSEQPRODCOM"] = $this->util->formatNrSeqProdCom($nrSeqProdComPai);
                } 
                elseif($subItem["price"] == 0){

                    $validateProd = $this->productFactory->getObservation($subItem['externalId']);

                    // if(empty($pruductSimpleTemplate['TXPRODCOMVEN'])){
                    //     $pruductSimpleTemplate['TXPRODCOMVEN'] = ' ';
                    // }
                    
                    if(empty($validateProd)) {
                        $txProdComVenPai .= " ".strtoupper("{$subItem['name']};");
                    } else {
                        $txProdComVenPai .= " ".strtoupper("{$validateProd['DSOCORR']};");
                        array_push($pruductSimpleTemplate['ITEM_OBSERVATION'], [
                            'CDFILIAL'        => $generalParams["CDFILIAL"],
                            "NRVENDAREST"     => $order["NRVENDAREST"],
                            "NRCOMANDA"       => $order["NRCOMANDA"],
                            'NRPRODCOMVEN'    => $nrSeqProdComPai,
                            'CDGRPOCOR'       => $validateProd['CDGRPOCOR'],
                            'CDOCORR'         => $validateProd['CDOCORR'],
                            'NRPRODCOMVENOBS' => null,
                            'QTPRODCOMVENOBS' => $subItem['quantity']
                        ]);
                    }
                }
            }
            
            $pruductSimpleTemplate["TXPRODCOMVEN"] = $txProdComVenPai;
            
            array_push($productItems, $pruductSimpleTemplate);
            
        }
        return $productItems;
    }
    
    private function processProductWithoutExtCode($generalParams, $product, $order, &$ITCOMANDAVEN_SEQ)
    {
        $productItems = [];

        for ($i = 0 ; $i < $product["quantity"]; $i++) {
            $nrSeqProdComPai = str_pad((string) ($ITCOMANDAVEN_SEQ + 1), 6, '0', STR_PAD_LEFT);
            
            foreach ($product["subItems"] as $subItem) {
            
                if($subItem["price"] > 0) {
            
                    if (!isset($subItem["externalId"]) || empty($subItem["externalId"])) {
                        throw new ParamsException("
                            O produto {$subItem['name']} não possui código externo e ele é obrigatório.<br>
                            Verifique no Odhen Retail o código externo deste produto e faça a correção na plataforma Anota Aí.<br>");
                    }
                    
                    $validateProd = $this->productFactory->getProduct($subItem["externalId"]);
                    
                    if (!isset($validateProd) || empty($validateProd)) {
                        throw new ParamsException("
                            O produto {$subItem['name']} não possui o código externo {$subItem['externalId']} cadastrado no Sistema Teknisa .<br>
                            Verifique no Odhen Retail o código externo deste produto e faça as devidas correções.<br>");
                    }

                    for($y = 0; $y < $subItem["quantity"]; $y++) {
                        $subItemDetails = array_merge($validateProd, $subItem);
                        
                        $subItemDetails["NRPRODCOMVEN"] = str_pad((string) ($ITCOMANDAVEN_SEQ + 1), 6, '0', STR_PAD_LEFT);
                        $ITCOMANDAVEN_SEQ++;
                        
                        $txProdComVenIt = "";
                        if(!empty($subItemDetails["observation"])){
                            $txProdComVenIt = " ".$subItemDetails["observation"].";";
                        }
                        
                        $subItemReturn = [
                            "CDFILIAL"          => $generalParams["CDFILIAL"],
                            "NRVENDAREST"       => $order["NRVENDAREST"],
                            "NRCOMANDA"         => $order["NRCOMANDA"],
                            "NRPRODCOMVEN"      => $subItemDetails["NRPRODCOMVEN"],
                            "CDPRODUTO"         => $subItemDetails["CDPRODUTO"],
                            "QTPRODCOMVEN"      => 1,
                            "VRPRECCOMVEN"      => $subItemDetails["price"], //bcdiv($subItemDetails["price"], 100, 2),
                            "TXPRODCOMVEN"      => $txProdComVenIt,
                            "IDSTPRCOMVEN"      => $generalParams["IDCONTROPROD"] === "S" ? "1" : "3",
                            "VRDESCCOMVEN"      => 0,
                            "NRLUGARMESA"       => "01",
                            "IDPRODIMPFIS"      => "N",
                            "CDLOJA"            => $generalParams["CDLOJA"],
                            "VRACRCOMVEN"       => 0,
                            "NRSEQPRODCOM"      => $this->util->formatNrSeqProdCom($nrSeqProdComPai),
                            "NRSEQPRODCUP"      => $generalParams["NRSEQPRODCUP"],
                            "VRPRECCLCOMVEN"    => 0,
                            "CDCAIXACOLETOR"    => $generalParams["CDCAIXA"],
                            "CDPRODPROMOCAO"    => $subItemDetails["CDPRODUTO"],
                            "CDVENDEDOR"        => $generalParams["CDVENDEDOR"],
                            "NRINSCRCONS"       => "",
                            "DSOBSPEDDIGCMD"    => null,
                            "ITEM_OBSERVATION"  => [] //
                        ];
                        
                        array_push($productItems, $subItemReturn);
                    }
                    $pruductSimpleTemplate["NRSEQPRODCOM"] = $this->util->formatNrSeqProdCom($nrSeqProdComPai);
                } 
                elseif($subItem["price"] == 0){
                
                // Observações tipo O e R
                    if(!isset($subItem['externalId']) || empty($subItem['externalId']))
                        throw new Exception\ParamsException("
                            O subitem {$subItem['name']}, do item {$product['name']}, foi cadastrado sem preço e sem código externo. Apesar do preço não ser uma exigência (ver abaixo), o código externo é obrigatório para todos os subitems.<br>
                            Caso o subitem seja uma observação (por exemplo, 'Sem salada' ou 'Ao ponto'), ele não deve ter preço e será necessário atrelar o código externo deste subitem, no formato Código Grupo de Observação|Código de Observação (xx|yy).<br>
                            Caso o subitem seja um acréscimo (por exemplo, 'Dobro de Queijo'), será necessário atrelar um valor igual ou superior a R$ 0,01 e atrelar o código externo deste subitem, no formato 'x.xx.xx.xxx.xx' ou no formato Código Grupo de Observação|Código de Observação (xx|yy).<br>
                            O código externo deve ser igual ao cadastrado previamente no Retail.<br>
                            Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual do Delivery Direto.");

                    if(!strpos($subItem['externalId'], "|"))
                        throw new Exception\ParamsException("
                            O subitem {$subItem['name']}, do item {$product['name']}, foi cadastrado sem preço e não existe um subitem cadastrado no Retail com o código externo informado: {$subItem['externalId']}.<br>
                            Caso o subitem seja uma observação (por exemplo, 'Sem salada' ou 'Ao ponto'), será necessário corrigir o código externo deste subitem. O formato correto é Código Grupo de Observação|Código de Observação (xx|yy).<br>
                            Caso o subitem seja um acréscimo (por exemplo, 'Dobro de Queijo'), será necessário corrigir o valor do subitem, que deve ser igual ou superior a R$ 0,01.<br>
                            O código externo e o valor do subitem (caso tenha) devem ser iguais ao cadastrado previamente no Retail.<br>
                            Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual do Delivery Direto.");

                    $validateProd = $this->productFactory->getObservation($subItem['externalId']);

                    if(empty($validateProd)) {
                        $txProdComVenPai .= " ".strtoupper("{$subItem['name']};");
                    } else {
                        $txProdComVenPai .= " ".strtoupper("{$validateProd['DSOCORR']};");
                        array_push($pruductSimpleTemplate['ITEM_OBSERVATION'], [
                            'CDFILIAL'        => $generalParams["CDFILIAL"],
                            "NRVENDAREST"     => $order["NRVENDAREST"],
                            "NRCOMANDA"       => $order["NRCOMANDA"],
                            'NRPRODCOMVEN'    => $nrSeqProdComPai,
                            'CDGRPOCOR'       => $validateProd['CDGRPOCOR'],
                            'CDOCORR'         => $validateProd['CDOCORR'],
                            'NRPRODCOMVENOBS' => null,
                            'QTPRODCOMVENOBS' => $subItem['quantity']
                        ]);
                    }
                }
            }
        }
        
        return $productItems;
    }

    public function insertProduct ($product)
    {
        $this->productFactory->insertProduct($product);
    }

    public function insertObservation($observation)
    {
        $this->productFactory->insertObservation($observation);
    }
}